//////////////////////////////////////////////////////////////////////////////////
// Company: 武汉芯路恒科技有限公司
// Engineer: 小梅哥团队
// Web: www.corecourse.cn
// 
// Create Date: 2021/08/20 00:00:00
// Design Name: uart_rtc8563_hex8
// Module Name: uart_rtc8563_hex8
// Project Name: uart_rtc8563_hex8
// Target Devices: EP4CE10F17C8
// Tool Versions: Quartus II 13.0 64-bit
// Description: 串口输入的时间调整模块
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////

module uart_adjust_time(
    //System clock reset
    input clk, //系统时钟输入，50M
    input reset_n, //复位信号输入，低有效
    
    input [7:0]uart_rx_data   , //串口接收数据
    input uart_data_valid, //串口接收数据有效标识
    
    output set_time       , //设置时间使能
    output  [23:0]set_time_data  , //设置时间数据
    output  reg set_date       , //设置日期使能
    output  [31:0]set_date_data    //设置日期数据
);
	parameter DLY_CNT_MAX = 5_000_000;

//调整时间的串口接收素数据协议命令格式
//           header         + 年+月+日+周+时+分+秒
//0x01 + 0x03 + 0x02 + 0x04 + 年+月+日+周+时+分+秒
	reg [87:0]rx_cmd_data;
	wire cmd_checked;     //如果为真，表示检测到帧头，开始将收到的时间校准到时钟芯片
	reg cmd_checked_dly1;
	reg [25:0]dly_cnt;
	wire dly_done;

	always @ (posedge clk or negedge reset_n)
	begin
    if (!reset_n)
      rx_cmd_data <= 'd0;
    else if (uart_data_valid)
      rx_cmd_data <= {rx_cmd_data[79:0], uart_rx_data};
    else
      rx_cmd_data <= rx_cmd_data;
	end

	assign cmd_checked = (rx_cmd_data[87:56] == 32'h01030204);

  //接收到正确的时间调整命令后，先对时间进行调整，然后0.1s后对日期进行调整更新
	always @ (posedge clk or negedge reset_n)
	begin
    if (!reset_n)
      cmd_checked_dly1 <= 1'b0;
    else
      cmd_checked_dly1 <= cmd_checked;
	end  
  
	assign set_time = (cmd_checked && (!cmd_checked_dly1)); //时间调整使能
	assign set_time_data = rx_cmd_data[23:0];	              //时间调整数据，00:23:20	

  //时间调整使能后，等待0.1s
  //0.1s时间计数器
	always @ (posedge clk or negedge reset_n)
	begin
    if (!reset_n)
      dly_cnt <= 'd0;
    else if (set_time)
      dly_cnt <= DLY_CNT_MAX;
    else if (dly_cnt > 0)
      dly_cnt <= dly_cnt - 1'd1;
    else
      dly_cnt <= dly_cnt;
	end

	always @ (posedge clk or negedge reset_n)
	begin
    if (!reset_n)
      set_date <= 1'b0;
    else if (dly_cnt == 1)
      set_date <= 1'b1;
    else
      set_date <= 1'b0;
	end	

	assign set_date_data = {rx_cmd_data[55:40], rx_cmd_data[31:24], rx_cmd_data[39:32]};//日期调整数据，19-05-01

endmodule
