
`include "disp_parameter_cfg.v"

	// Resolution_800x480
	// H_Total_Time 	=	12'd1056 	V_Total_Time 	 =	12'd525
	// H_Data_Time 	=	12'd800		V_Data_Time 	 =	12'd480
	// H_Right_Border =	12'd0			V_Bottom_Border = 12'd8
	// H_Front_Porch 	=	12'd40		V_Front_Porch 	 =	12'd2
	// H_Sync_Time 	=	12'd128		V_Sync_Time 	 =	12'd2
	// H_Back_Porch 	=	12'd88		V_Back_Porch 	 =	12'd25
	// H_Left_Border 	=	12'd0			V_Top_Border 	 =	12'd8

	

module VGA_CTRL_1(
	// 系统相关
	clk_vga,
	rst_n,
	// 图像输入数据，比如摄像头采集数据
	data_in,
	// 控制图像输出位置
	hcount,
	vcount,
	// VGA相关信号
	VGA_HS,	// 行同步		
	VGA_VS,	//	场同步
	VGA_BLK,	// 数据使能信号
	VGA_CLK,	// vga时钟
	VGA_DATA // 输出图像数据
);

	input 			clk_vga;
	input 			rst_n;
	input [23:0] 	data_in;	// RGB888
	output [11:0] 	hcount;
	output [11:0] 	vcount;
	output 			VGA_HS;	
	output 			VGA_VS;	
	output 			VGA_BLK;
	output 			VGA_CLK;
	output [23:0]	VGA_DATA;
	
	// 行时序节点 
	localparam HS_End 		= `H_Sync_Time; // HS_Begin + Sync_Pulse
	localparam Hdat_Begin 	= `H_Sync_Time+`H_Back_Porch+`H_Left_Border; // HS_End + Back_Porch + Left_Border
	localparam Hdat_End 		= `H_Sync_Time+`H_Back_Porch+`H_Left_Border+`H_Data_Time;// Hdat_Begin + Visible_area
	localparam Hsync_End 	= `H_Total_Time;// Hdat_End + Right_Border + Front_Porch
	// 场时序节点
	localparam VS_End 		= `V_Sync_Time;	// VS_Begin + Sync_Pulse 
	localparam Vdat_Begin 	= `V_Sync_Time+`V_Back_Porch+`V_Top_Border; // VS_End + Back_Porch + Top_Border
	localparam Vdat_End 		= `V_Sync_Time+`V_Back_Porch+`V_Top_Border+`V_Data_Time;// Vdat_Begin + Visible_area
	localparam Vsync_End 	= `V_Total_Time;//Vdat_End  + Bottom_Border + Front_Porch
	
	assign VGA_CLK = ~clk_vga;
	
	// 行同步控制 VGA_HS
	reg [11:0] h_cnt;
	
	always @ (posedge clk_vga or negedge rst_n)
		if (!rst_n)
			h_cnt <= 12'd0;
		else if (h_cnt == Hsync_End - 1'd1)
			h_cnt <= 12'd0;
		else 
			h_cnt <= h_cnt + 1'd1;

	assign VGA_HS = (h_cnt >= HS_End) ? 1'b1 : 1'b0;
			
			
	// 场同步控制 VGA_VS
	reg [11:0] v_cnt;

	always @ (posedge clk_vga or negedge rst_n)
		if (!rst_n)
			v_cnt <= 12'd0;
		else if (h_cnt == Hsync_End - 1'd1) begin
			if (v_cnt == Vsync_End - 1'd1)
				v_cnt <= 12'd0;
			else 
				v_cnt <= v_cnt + 1'd1;
		end
		else 
			v_cnt <= v_cnt;
			
			
	assign VGA_VS = (v_cnt >= VS_End) ? 1'b1 : 1'b0;
	
	// 数据使能信号 VGA_BLK
	
	assign VGA_BLK = ((h_cnt >= Hdat_Begin) && (h_cnt < Hdat_End)&&
							(v_cnt >= Vdat_Begin) && (v_cnt < Vdat_End)) ? 1'b1 : 1'b0;
	
	// 输出数据 VGA_DATA
	
	assign VGA_DATA = VGA_BLK ? data_in : 24'd0;
	
	// 确定图像位置
	
	assign hcount = VGA_BLK ? (h_cnt - Hdat_Begin) : 12'd0;
	assign vcount = VGA_BLK ? (v_cnt - Vdat_Begin) : vcount;
	
endmodule 