`include "disp_parameter_cfg.v"

module VGA_TOP(
	clk,
	rst_n,
	VGA_HS,	
	VGA_VS,	
//	VGA_BLK,	
//	VGA_CLK,	
	VGA_DATAA
);

	input 			clk;
	input 			rst_n;
	output 			VGA_HS;	
	output 			VGA_VS;	
//	output 			VGA_BLK;
//	output 			VGA_CLK;
	output [7:0]	VGA_DATAA;
	
	wire [23:0] VGA_DATA_reg;
	assign VGA_DATAA = {VGA_DATA_reg[23:21],VGA_DATA_reg[15:13],VGA_DATA_reg[7:6]};
	
	wire 			VGA_BLK;
	wire 			VGA_CLK;

	wire clk_vga;
	wire locked;
	reg [23:0] data_in;
	wire [10:0] hcount;
	wire [10:0] vcount;
	
	reg en; // 切换使能信号
	reg [31:0] cnt;
	
	// 颜色代码
	localparam	BLACK		= 24'h000000, // 黑色
					BLUE		= 24'h0000FF, // 蓝色
					RED		= 24'hFF0000, // 红色
					PURPPLE	= 24'hFF00FF, // 紫色
					GREEN		= 24'h00FF00, // 绿色
					CYAN		= 24'h00FFFF, // 青色
					YELLOW	= 24'hFFFF00, // 黄色
					WHITE		= 24'hFFFFFF; // 白色

	/*************************彩条划分****************************/
	// 横彩条8块区域， 两列四行
//	wire h0_act = (hcount >= 0  ) && (hcount < 400); // 正在扫描第0列 
//	wire h1_act = (hcount >= 400) && (hcount < 800); // 正在扫描第1列 
//	wire v0_act = (vcount >= 0  ) && (vcount < 120); // 正在扫描第0行
//	wire v1_act = (vcount >= 120) && (vcount < 240); // 正在扫描第1行  
//	wire v2_act = (vcount >= 240) && (vcount < 360); // 正在扫描第2行 
//	wire v3_act = (vcount >= 360) && (vcount < 480); // 正在扫描第3行 
	
/******可调分辨率********/	
	wire h0_act = (hcount >= 0  ) && (hcount < `H_Data_Time/2); 				// 正在扫描第0列 
	wire h1_act = (hcount >= `H_Data_Time/2) && (hcount < `H_Data_Time); 	// 正在扫描第1列 
	wire v0_act = (vcount >= 0  ) && (vcount < `V_Data_Time/4); 				// 正在扫描第0行
	wire v1_act = (vcount >= `V_Data_Time/4) && (vcount < `V_Data_Time/2); 	// 正在扫描第1行  
	wire v2_act = (vcount >= `V_Data_Time/2) && (vcount < `V_Data_Time/4*3); // 正在扫描第2行 
	wire v3_act = (vcount >= `V_Data_Time/4*3) && (vcount < `V_Data_Time); 	// 正在扫描第3行 
	
	wire h0_v0_act = h0_act & v0_act; // 第0列0行像素块被扫描中
	wire h0_v1_act = h0_act & v1_act; // 第0列1行像素块被扫描中
	wire h0_v2_act = h0_act & v2_act; // 第0列2行像素块被扫描中
	wire h0_v3_act = h0_act & v3_act; // 第0列3行像素块被扫描中
	
	wire h1_v0_act = h1_act & v0_act; // 第1列0行像素块被扫描中
	wire h1_v1_act = h1_act & v1_act; // 第1列1行像素块被扫描中
	wire h1_v2_act = h1_act & v2_act; // 第1列2行像素块被扫描中
	wire h1_v3_act = h1_act & v3_act; // 第1列3行像素块被扫描中

	// 竖彩条8块区域， 两行四列
//	wire H0_act = (hcount >= 0  ) && (hcount < 200); // 正在扫描第0列 
//	wire H1_act = (hcount >= 200) && (hcount < 400); // 正在扫描第1列 
//	wire H2_act = (hcount >= 400) && (hcount < 600); // 正在扫描第2列
//	wire H3_act = (hcount >= 600) && (hcount < 800); // 正在扫描第3列  
//	wire V0_act = (vcount >= 0  ) && (vcount < 240); // 正在扫描第0行 
//	wire V1_act = (vcount >= 240) && (vcount < 480); // 正在扫描第1行 
	
/******可调分辨率********/	
	wire H0_act = (hcount >= 0  ) && (hcount < `H_Data_Time/4); 				// 正在扫描第0列 
	wire H1_act = (hcount >= `H_Data_Time/4) && (hcount < `H_Data_Time/2); 	// 正在扫描第1列 
	wire H2_act = (hcount >= `H_Data_Time/2) && (hcount < `H_Data_Time/4*3); // 正在扫描第2列
	wire H3_act = (hcount >= `H_Data_Time/4*3) && (hcount < `H_Data_Time); 	// 正在扫描第3列  
	wire V0_act = (vcount >= 0  ) && (vcount < `V_Data_Time/2); 				// 正在扫描第0行 
	wire V1_act = (vcount >= `V_Data_Time/2) && (vcount < `V_Data_Time); 	// 正在扫描第1行 
	
	wire V0_H0_act = V0_act & H0_act; // 第0行0列像素块被扫描中
	wire V0_H1_act = V0_act & H1_act; // 第0行1列像素块被扫描中
	wire V0_H2_act = V0_act & H2_act; // 第0行2列像素块被扫描中
	wire V0_H3_act = V0_act & H3_act; // 第0行3列像素块被扫描中
	
	wire V1_H0_act = V1_act & H0_act; // 第1行0列像素块被扫描中
	wire V1_H1_act = V1_act & H1_act; // 第1行1列像素块被扫描中
	wire V1_H2_act = V1_act & H2_act; // 第1行2列像素块被扫描中
	wire V1_H3_act = V1_act & H3_act; // 第1行3列像素块被扫描中
	
	// 横竖彩条切换赋值
	always@(posedge clk_vga)
		if (en ==  1) begin // 横彩条
			case ({h1_v3_act,h1_v2_act,h1_v1_act,h1_v0_act,
					h0_v3_act,h0_v2_act,h0_v1_act,h0_v0_act})
				8'b0000_0001:data_in <= BLACK	 ;
				8'b0000_0010:data_in <= BLUE	 ;
				8'b0000_0100:data_in <= RED	 ;
				8'b0000_1000:data_in <= PURPPLE;
				8'b0001_0000:data_in <= GREEN	 ;
				8'b0010_0000:data_in <= CYAN	 ;
				8'b0100_0000:data_in <= YELLOW ;
				8'b1000_0000:data_in <= WHITE	 ;
			endcase
		end
		else if (en == 0) begin // 竖彩条
			case ({V1_H3_act,V1_H2_act,V1_H1_act,V1_H0_act,
					V0_H3_act,V0_H2_act,V0_H1_act,V0_H0_act})
				8'b0000_0001:data_in <= BLACK	 ;
				8'b0000_0010:data_in <= BLUE	 ;
				8'b0000_0100:data_in <= RED	 ;
				8'b0000_1000:data_in <= PURPPLE;
				8'b0001_0000:data_in <= GREEN	 ;
				8'b0010_0000:data_in <= CYAN	 ;
				8'b0100_0000:data_in <= YELLOW ;
				8'b1000_0000:data_in <= WHITE	 ;
			endcase
		end
	
	// 切换使能信号 en

	always @ (posedge clk or negedge rst_n)
		if (!rst_n)
			cnt <= 32'd0;
		else if (cnt == 32'd99_999_999)
			cnt <= 32'd0;
		else 
			cnt <= cnt + 1'b1;
			
	always @ (posedge clk or negedge rst_n)
		if (!rst_n)
			en <= 0;
		else if (cnt >= 32'd49_999_999)
			en <= 1;
		else 
			en <= 0;
	
//	VGA_CTRL VGA_CTRL(
//		.clk_33m	(clk_vga	),
//		.rst_n	(locked	),
//		.data_in	(data_in	),
//		.hcount	(hcount	),
//		.vcount	(vcount	),
//		.VGA_HS	(VGA_HS	),
//		.VGA_VS	(VGA_VS	),
//		.VGA_BLK	(VGA_BLK	),
//		.VGA_CLK	(VGA_CLK	),
//		.VGA_DATA(VGA_DATA)
//	);

/******可调分辨率********/	
	VGA_CTRL_1 VGA_CTRL_1(
		.clk_vga	(clk_vga	),
		.rst_n	(rst_n	),
		.data_in	(data_in	),
		.hcount	(hcount	),
		.vcount	(vcount	),
		.VGA_HS	(VGA_HS	),
		.VGA_VS	(VGA_VS	),
		.VGA_BLK	(VGA_BLK	),
		.VGA_CLK	(VGA_CLK	),
		.VGA_DATA(VGA_DATA_reg)
	);

	vga_33m	vga_33m_inst (
		.areset ( !rst_n ),
		.inclk0 ( clk ),
		.c0 ( clk_vga ),
		.locked ( locked )
	);	

endmodule 